/*
Copyright (c) 2015 Red Hat, Inc.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

  http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

package org.ovirt.engine.sdk4.services;

import java.io.IOException;
import java.math.BigInteger;
import java.util.List;
import org.ovirt.engine.sdk4.Request;
import org.ovirt.engine.sdk4.Response;
import org.ovirt.engine.sdk4.Service;
import org.ovirt.engine.sdk4.builders.HostBuilder;
import org.ovirt.engine.sdk4.types.Host;

public interface HostsService extends Service {
    /**
     * Creates a new host and adds it to the database. The host is created based on the properties of the `host`
     * parameter. The `name`, `address` `rootPassword` properties are required.
     */
    public interface AddRequest extends Request<AddRequest, AddResponse> {
        /**
         * When set to `true` it means this host should deploy also hosted
         * engine components. Missing value is treated as `true` i.e deploy.
         * Omitting this parameter means `false` and will perform no operation
         * in hosted engine area.
         */
        AddRequest deployHostedEngine(Boolean deployHostedEngine);
        AddRequest host(Host host);
        /**
         * The host definition from which to create the new host is passed as parameter, and the newly created host
         * is returned.
         */
        AddRequest host(HostBuilder host);
        /**
         * When set to `true` it means this host should un-deploy hosted engine
         * components and this host will not function as part of the High
         * Availability cluster. Missing value is treated as `true` i.e un-deploy.
         * Omitting this parameter means `false` and will perform no operation
         * in hosted engine area.
         */
        AddRequest undeployHostedEngine(Boolean undeployHostedEngine);
    }
    
    /**
     * Creates a new host and adds it to the database. The host is created based on the properties of the `host`
     * parameter. The `name`, `address` `rootPassword` properties are required.
     */
    public interface AddResponse extends Response {
        /**
         * The host definition from which to create the new host is passed as parameter, and the newly created host
         * is returned.
         */
        Host host();
    }
    
    /**
     * Creates a new host and adds it to the database. The host is created based on the properties of the `host`
     * parameter. The `name`, `address` `rootPassword` properties are required.
     */
    AddRequest add();
    
    public interface ListRequest extends Request<ListRequest, ListResponse> {
        /**
         * Indicates if the search performed using the `search` parameter should be performed taking case into
         * account. The default value is `true`, which means that case is taken into account. If you want to search
         * ignoring case set it to `false`.
         */
        ListRequest caseSensitive(Boolean caseSensitive);
        /**
         * Indicates if the results should be filtered according to the permissions of the user.
         */
        ListRequest filter(Boolean filter);
        /**
         * Sets the maximum number of hosts to return. If not specified all the hosts are returned.
         */
        ListRequest max(Integer max);
        /**
         * Sets the maximum number of hosts to return. If not specified all the hosts are returned.
         */
        ListRequest max(Long max);
        /**
         * Sets the maximum number of hosts to return. If not specified all the hosts are returned.
         */
        ListRequest max(BigInteger max);
        /**
         * A query string used to restrict the returned hosts.
         */
        ListRequest search(String search);
    }
    
    public interface ListResponse extends Response {
        List<Host> hosts();
    }
    
    ListRequest list();
    
    HostService hostService(String id);
    /**
     * Service locator method, returns individual service on which the URI is dispatched.
     */
    Service service(String path);
}

